@@ -1,6 +1,6 @@
 /* MPFR internal constant FP numbers
 
-Copyright 2005 Free Software Foundation.
+Copyright 2005, 2006 Free Software Foundation, Inc.
 
 This file is part of the MPFR Library.
 
@@ -16,15 +16,12 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Place, Fifth Floor, Boston,
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
 
 static const mp_limb_t __gmpfr_limb1[1] = {MPFR_LIMB_HIGHBIT};
-const MPFR_THREAD_ATTR mpfr_t __gmpfr_one =
-  {{2, MPFR_SIGN_POS, 1, (mp_limb_t*)__gmpfr_limb1}};
-const MPFR_THREAD_ATTR mpfr_t __gmpfr_two =
-  {{2, MPFR_SIGN_POS, 2, (mp_limb_t*)__gmpfr_limb1}};
-const MPFR_THREAD_ATTR mpfr_t __gmpfr_four =
-  {{2, MPFR_SIGN_POS, 3, (mp_limb_t*)__gmpfr_limb1}};
+const mpfr_t __gmpfr_one = {{2, MPFR_SIGN_POS, 1, (mp_limb_t*)__gmpfr_limb1}};
+const mpfr_t __gmpfr_two = {{2, MPFR_SIGN_POS, 2, (mp_limb_t*)__gmpfr_limb1}};
+const mpfr_t __gmpfr_four ={{2, MPFR_SIGN_POS, 3, (mp_limb_t*)__gmpfr_limb1}};
